/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "domstubs.idl"

interface nsITCPSocketInternal;
interface nsIDOMWindow;

// Interface to allow the content process socket to reach the IPC bridge.
[scriptable, uuid(bdc91763-e9a1-4122-9c2f-8f17505c8c7a)]
interface nsITCPSocketChild : nsISupports
{
  // Tell the chrome process to open a corresponding connection with the given parameters
  [implicit_jscontext]
  void open(in nsITCPSocketInternal socket, in DOMString host,
            in unsigned short port, in boolean ssl, in DOMString binaryType,
            in nsIDOMWindow window, in jsval socketVal);

  // Tell the chrome process to perform equivalent operations to all following methods
  [implicit_jscontext] void send(in jsval data, in unsigned long byteOffset, in unsigned long byteLength);
  void resume();
  void suspend();
  void close();
};
