/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */


#include "nsISupports.idl"

interface nsILoginInfo;

[scriptable, uuid(8a59ea3d-b8d0-48af-a3e2-63e27a02cde7)]

interface nsILoginManagerIEMigrationHelper : nsISupports {
    /**
     * Takes a login provided from nsIEProfileMigrator, migrates it to the
     * current login manager format, and adds it to the list of stored logins.
     *
     * @param aLogin
     *        The login to be migrated.
     *
     * Note: In some cases, multiple logins may be created from a single input
     *       when the format is ambigious.
     *
     */
    void migrateAndAddLogin(in nsILoginInfo aLogin);
};
