/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

/**
 * Corresponds to http://www.w3.org/TR/2004/REC-DOM-Level-3-Core-20040407
 */

#include "domstubs.idl"

[scriptable, uuid(0bbae65c-1dde-11d9-8c46-000a95dc234c)]
interface nsIDOMDOMStringList : nsISupports
{
  DOMString          item(in unsigned long index);
  readonly attribute unsigned long   length;
  boolean            contains(in DOMString str);
};
